module Releaf
  module Content::Node
    # Validator to test if node is valid, when created under given parrent node
    #
    # validator needs :for and :under options, both can be either array of classes
    # or single class.
    #
    # :for option specifies for which nodes validation should be applied
    #
    # :under option specifies under which nodes given node can be added
    #
    # @example
    #
    #   class Node < ActiveRecord::Base
    #     includes Releaf::Content::Node
    #     validates_with Releaf::Content::Node::ParentValidator, for: [Text, Book], under: Store
    #   end
    #
    class ParentValidator < ActiveModel::Validator

      def validate node
        @node = node
        node.errors.add(:content_type, 'invalid parent node') unless node_parent_valid?
        remove_instance_variable(:@node)
      end

      private

      def node_parent_valid?
        return true unless child_class_names.include? @node.content_type
        return parent_class_names.include? @node.parent.try(:content_type)
      end

      def child_class_names
        target_class_names :for
      end

      def parent_class_names
        target_class_names :under
      end

      def target_class_names target_type
        [options.fetch(target_type, [])].flatten.map(&:name)
      end

    end
  end
end
